package cmsutil;

import java.util.*;
import java.io.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.opencms.core.*;
import com.opencms.file.*;
import com.opencms.defaults.*;
import com.opencms.template.*;
import com.opencms.util.*;
import com.opencms.workplace.*;
import com.opencms.template.cache.*;

public class CmsPrintPage extends CmsXmlTemplate implements I_CmsConstants,I_CmsWpConstants {

    public CmsCacheDirectives getCacheDirectives(CmsObject cms, String templateFile, String elementName, Hashtable parameters, String templateSelector) {

        return new CmsCacheDirectives(false);
    }

    private void getSubTemplate(CmsObject cms, String xmlTemplate, CmsElementDefinitionCollection eldefs) throws CmsException {
	
		CmsXmlTemplateFile xmlTemplateDocument = new CmsXmlTemplateFile(cms, xmlTemplate);
		Vector subtemplates = xmlTemplateDocument.getAllSubElementDefinitions();
		int numSubtemplates = subtemplates.size();
		for(int i = 0; i < numSubtemplates; i++) {
			String elName = (String)subtemplates.elementAt(i);
			String className = null;
			String templateName = null;
			String templateSelector = null;

			if(xmlTemplateDocument.hasSubtemplateClass(elName)) {
				className = xmlTemplateDocument.getSubtemplateClass(elName);
			}
			if(xmlTemplateDocument.hasSubtemplateFilename(elName)) {
				templateName = xmlTemplateDocument.getSubtemplateFilename(elName);
			}
			if(xmlTemplateDocument.hasSubtemplateSelector(elName)) {
				templateSelector = xmlTemplateDocument.getSubtemplateSelector(elName);
			}
			Hashtable templateParameters = xmlTemplateDocument.getParameters(elName);
			if(className != null || templateName != null || templateSelector != null || templateParameters.size() > 0) {
				if(className == null){
					className = "com.opencms.template.CmsXmlTemplate";
				}
				if(templateName != null){
					templateName = Utils.mergeAbsolutePath(xmlTemplate, templateName);
				}
				CmsElementDefinition elDef = new CmsElementDefinition(elName, className, templateName, templateSelector, templateParameters);
				eldefs.add(elDef);
			}
		}
	}
	
	private String getBodyTemplateName(CmsObject cms, CmsXmlControlFile doc) throws CmsException {
		return "body";
	}
	
	private Hashtable buildHashtable(CmsObject cms, String xmlTemplate) throws CmsException {
		
		Hashtable parameters = new Hashtable();
		CmsXmlControlFile controlFile = new CmsXmlControlFile(cms, xmlTemplate);
		
		CmsElementDefinitionCollection eldefs = controlFile.getElementDefinitionCollection();
		CmsElementDefinitionCollection subtemplateDefinitions = new CmsElementDefinitionCollection();
		try {
            Enumeration els = eldefs.getAllElementNames();
            while(els.hasMoreElements()){
                String currentElement = (String)els.nextElement();
                CmsElementDefinition eldef = eldefs.get(currentElement);
                parameters.put(currentElement +"._TEMPLATE_", eldef.getTemplateName());
				getSubTemplate(cms, eldef.getTemplateName(), subtemplateDefinitions);
                parameters.put(currentElement +"._CLASS_", eldef.getClassName());
                if(eldef.getTemplateSelector()!= null) {
                    parameters.put(currentElement +"._TEMPLATESELECTOR_", eldef.getTemplateSelector());
                } else {
                    parameters.put(currentElement +"._TEMPLATESELECTOR_", "default");
                }
            }
        } catch(NullPointerException exc) {
            // no body-element found - ignoring
        }
		parameters.put("_ELDEFS_", subtemplateDefinitions);
		return parameters;
	}
	
    public byte[] getContent(CmsObject cms, String templateFile, String elementName, Hashtable parameters, String templateSelector) throws CmsException {

        // template file
        CmsXmlTemplateFile template = getOwnTemplateFile(cms, templateFile, elementName, parameters, templateSelector);
		CmsSession session = (CmsSession) cms.getRequestContext().getSession(true);
		CmsLanguage language = CmsLanguage.getLanguage(cms, session, parameters);
        // get parameters
        String requestedUri = (String) parameters.get("requesteduri");
        requestedUri = (requestedUri!=null)?requestedUri.trim():"";
		template.setData("url", requestedUri);
		String servletPath = cms.getRequestContext().getRequest().getServletUrl();
		requestedUri = requestedUri.substring(servletPath.length());
	
		CmsXmlControlFile controlFile = new CmsXmlControlFile(cms, requestedUri);
		String element = getBodyTemplateName(cms, controlFile);
		String elementTemplate = controlFile.getElementTemplate(element);
		String elementClassName = controlFile.getElementClass(element);
		CmsXmlTemplate bodyElementClassObject = (CmsXmlTemplate) CmsTemplateClassManager.getClassInstance(cms, elementClassName);
	
		Hashtable URIparams = buildHashtable(cms, requestedUri);
		// add uripage parameters		
		Enumeration en = parameters.keys();
		while(en.hasMoreElements()){
			 Object key = en.nextElement();
			 if (!URIparams.containsKey(key)) {
				 URIparams.put(key, parameters.get(key));
			 }
		}
		byte[] content = bodyElementClassObject.getContent(cms, elementTemplate, element, URIparams, null);
		
		// set template selector
		String selector = language.getBodySelector();
        template.setData("PageContent", new String(content));
		return startProcessing(cms, template, elementName, parameters, templateSelector);
    }
}
